@@ -30,6 +30,10 @@
  *	http://www.radix.net/~cknudsen/gtimer/
  *
  * History:
+ *	02-Feb-1999	gtk 1.1 support
+ *	04-Jan-1999	Eek...  Fixed bug for "last month" reports when
+ *			run in Jan.
+ *	10-May-1998	Removed ifdef for GTK versions before 1.0.0
  *	05-Apr-1998	Added word wrap for report display
  *	04-Apr-1998	Added print
  *	28-Mar-1998	Finished adding support for HTML.
@@ -37,7 +41,7 @@
  *	26-Mar-1998	Added O_TRUNC option when saving to a file.
  *	26-Mar-1998	Added preliminary support for HTML output.
  *			Must save to a file and open with Netscape.
- *	22-Mar-1998	Added midnight_offset as an parameter to
+ *	22-Mar-1998	Added config_midnight_offset as an parameter to
  *			TaskGetAnnotationEntries.  Entries after midnight
  *			were showing up on the day after the hours were
  *			recorded.
@@ -54,7 +58,7 @@
  *			including hours worked optional.
  *	16-Mar-1998	Removed reference to sys_errlist[] for
  *			compatibility with glibc.
- *	13-Mar-1998	Use midnight_offset when figuring out what the
+ *	13-Mar-1998	Use config_midnight_offset when figuring out what the
  *			current date is.
  *	13-Mar-1998	Include seconds in reports
  *	12-Mar-1998	Remove toggle buttons for report type.  Now set
@@ -75,7 +79,9 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <pwd.h>
 #include <time.h>
 #include <memory.h>
@@ -101,9 +107,9 @@
 static int month_days[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
 static int lmonth_days[] = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
 
-extern TaskData **tasks;
-extern int num_tasks;
-extern int midnight_offset;
+extern TaskData **visible_tasks;
+extern int num_visible_tasks;
+extern int config_midnight_offset;
 extern GtkWidget *main_window;
 
 /* Report types */
@@ -251,11 +257,16 @@ GtkWidget *widget;
 gpointer data;
 {
   DisplayReportData *drd = (DisplayReportData *)data;
+  gtk_grab_remove ( drd->window );
   gtk_widget_destroy ( drd->window );
-  if ( drd->filesel )
+  if ( drd->filesel ) {
+    gtk_grab_remove ( drd->filesel );
     gtk_widget_destroy ( drd->filesel );
-  if ( drd->printwin )
+  }
+  if ( drd->printwin ) {
+    gtk_grab_remove ( drd->printwin );
     gtk_widget_destroy ( drd->printwin );
+  }
   free ( drd->text );
   free ( drd );
 }
@@ -265,6 +276,7 @@ GtkWidget *widget;
 gpointer data;
 {
   DisplayReportData *drd = (DisplayReportData *)data;
+  gtk_grab_remove ( drd->filesel );
   gtk_widget_destroy ( drd->filesel );
   drd->filesel = NULL;
 }
@@ -291,6 +303,7 @@ gpointer data;
   } else {
     write ( fd, drd->text, strlen ( drd->text ) );
     close ( fd );
+    gtk_grab_remove ( drd->filesel );
     gtk_widget_destroy ( drd->filesel );
     drd->filesel = NULL;
   }
@@ -353,6 +366,7 @@ gpointer data;
         "GTimer: Error", "Error printing", "Ok", NULL, NULL, NULL, NULL );
     } else {
       configSetAttribute ( CONFIG_PRINT, ptr );
+      gtk_grab_remove ( drd->printwin );
       gtk_widget_destroy ( drd->printwin );
       drd->printwin = NULL;
     }
@@ -366,6 +380,7 @@ gpointer data;
 {
   DisplayReportData *drd = (DisplayReportData *)data;
 
+  gtk_grab_remove ( drd->printwin );
   gtk_widget_destroy ( drd->printwin );
   drd->printwin = NULL;
 }
@@ -460,9 +475,7 @@ char *text;
   gtk_widget_set_usize ( textarea, 300, 200 );
   gtk_box_pack_start ( GTK_BOX ( hbox ),
     textarea, TRUE, TRUE, 2 );
-#if GTKVERSION >= 9900
   gtk_text_set_word_wrap ( GTK_TEXT ( textarea ), 1 );
-#endif
   gtk_widget_show ( textarea );
   gtk_widget_realize ( textarea );
   gtk_text_freeze ( GTK_TEXT ( textarea ) );
@@ -571,7 +584,7 @@ int is_last;
       entry = NULL;
     if ( include_annotations ) {
       anns = TaskGetAnnotationEntries ( seltasks[loop]->taskdata->task,
-        year, mon, mday, midnight_offset, &num_anns );
+        year, mon, mday, config_midnight_offset, &num_anns );
       for ( loop2 = 0; loop2 < num_anns; loop2++ ) {
         if ( type == REPORT_TYPE_DAILY )
           concat_annotation ( &seltasks[loop]->todays_annotations,
@@ -676,7 +689,7 @@ int is_last;
     s = ret % 60;
     switch ( format ) {
       case REPORT_OUTPUT_TEXT:
-        fprintf ( fp, "------\n" );
+        fprintf ( fp, "---------\n" );
         fprintf ( fp, "%3d:%02d:%02d - %s\n", h, m, s, "Total" );
         break;
       case REPORT_OUTPUT_HTML:
@@ -1116,6 +1129,7 @@ gpointer data;
     }
   }
 
+  gtk_grab_remove ( rd->window );
   gtk_widget_destroy ( rd->window );
 
   /* Generate the report...
@@ -1132,7 +1146,7 @@ gpointer data;
       NULL, NULL,
       NULL );
     free ( tempfile );
-    for ( loop = 0; loop < num_tasks; loop++ )
+    for ( loop = 0; loop < num_visible_tasks; loop++ )
       free ( seltasks[loop] );
     free ( seltasks );
     free ( rd->tasks );
@@ -1141,7 +1155,7 @@ gpointer data;
     return;
   }
   time ( &now );
-  now -= midnight_offset;
+  now -= config_midnight_offset;
   tm = localtime ( &now );
   switch ( range ) {
     case REPORT_RANGE_TODAY:
@@ -1156,7 +1170,7 @@ gpointer data;
       time_end = now;
       break;
     case REPORT_RANGE_LAST_MONTH:
-      if ( tm->tm_mon == 1 )
+      if ( tm->tm_mon == 0 )
         time_start = now - ( ONE_DAY * ( tm->tm_mday - 1 ) ) - ONE_DAY * 31;
       else if ( tm->tm_year % 4 == 0 )
         time_start = now - ( ONE_DAY * ( tm->tm_mday - 1 ) ) -
@@ -1238,6 +1252,7 @@ GtkWidget *widget;
 gpointer data;
 {
   ReportData *rd = (ReportData *) data;
+  gtk_grab_remove ( rd->window );
   gtk_widget_destroy ( rd->window );
   free ( rd->tasks );
   free ( rd->list_items );
@@ -1246,6 +1261,39 @@ gpointer data;
 
 
 
+static void select_all_callback ( widget, data )
+GtkWidget *widget;
+gpointer data;
+{
+  ReportData *rd = (ReportData *) data;
+#if GTK_VERSION < 10100
+  int loop;
+
+  for ( loop = 0; loop < rd->num_tasks; loop++ )
+    gtk_list_select_item ( GTK_LIST ( rd->task_list ), loop );
+#else
+  gtk_list_select_all ( GTK_LIST ( rd->task_list ) );
+#endif
+}
+
+
+
+static void select_none_callback ( widget, data )
+GtkWidget *widget;
+gpointer data;
+{
+  ReportData *rd = (ReportData *) data;
+#if GTK_VERSION < 10100
+  int loop;
+
+  for ( loop = 0; loop < rd->num_tasks; loop++ )
+    gtk_list_unselect_item ( GTK_LIST ( rd->task_list ), loop );
+#else
+  gtk_list_unselect_all ( GTK_LIST ( rd->task_list ) );
+#endif
+}
+
+
 
 
 static GtkWidget *create_time_menu ( rd )
@@ -1317,7 +1365,7 @@ report_type type;
   GtkWidget *report_window;
   GtkWidget *table;
   /*GtkTooltips *tooltips;*/
-  GtkWidget *label, *ok_button, *cancel_button, *time_menu, *output_menu,
+  GtkWidget *label, *button, *time_menu, *output_menu,
     *data_menu, *scrolled;
   ReportData *rd;
   GList *items = NULL;
@@ -1389,27 +1437,36 @@ report_type type;
   scrolled = gtk_scrolled_window_new ( NULL, NULL );
   gtk_widget_set_usize ( scrolled, 300, 200 );
   gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW (scrolled),
-    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
+    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS );
   gtk_box_pack_start ( GTK_BOX ( GTK_DIALOG (report_window)->vbox ),
     scrolled, TRUE, TRUE, 2 );
   gtk_widget_show ( scrolled );
 
   rd->task_list = gtk_list_new ();
-  rd->tasks = (TaskData **) malloc ( sizeof ( TaskData * ) * num_tasks );
-  rd->list_items = (GtkWidget **) malloc ( sizeof ( GtkWidget * ) * num_tasks );
-  rd->num_tasks = num_tasks;
-  for ( loop = 0; loop < num_tasks; loop++ ) {
+  rd->tasks = (TaskData **) malloc ( sizeof ( TaskData * ) *
+    num_visible_tasks );
+  rd->list_items = (GtkWidget **) malloc ( sizeof ( GtkWidget * ) *
+    num_visible_tasks );
+  rd->num_tasks = num_visible_tasks;
+  for ( loop = 0; loop < num_visible_tasks; loop++ ) {
     rd->list_items[loop] =
-      gtk_list_item_new_with_label ( tasks[loop]->task->name );
+      gtk_list_item_new_with_label ( visible_tasks[loop]->task->name );
     items = g_list_append ( items, rd->list_items[loop] );
-    rd->tasks[loop] = tasks[loop];
+    rd->tasks[loop] = visible_tasks[loop];
     gtk_widget_show ( rd->list_items[loop] );
   }
+
+#if GTK_VERSION < 10100
   gtk_container_add (GTK_CONTAINER (scrolled), rd->task_list );
+#else
+  gtk_scrolled_window_add_with_viewport ( GTK_SCROLLED_WINDOW ( scrolled ),
+    rd->task_list );
+#endif
+
   gtk_list_append_items ( GTK_LIST ( rd->task_list ), items );
   gtk_list_set_selection_mode ( GTK_LIST ( rd->task_list ),
     GTK_SELECTION_MULTIPLE );
-  for ( loop = 0; loop < num_tasks; loop++ ) {
+  for ( loop = 0; loop < num_visible_tasks; loop++ ) {
     gtk_list_select_item ( GTK_LIST ( rd->task_list ), loop );
   }
   gtk_widget_show ( rd->task_list );
@@ -1419,26 +1476,43 @@ report_type type;
 
   gtk_box_set_homogeneous ( GTK_BOX ( GTK_DIALOG (report_window)->action_area ),
     1 );
-  ok_button = gtk_button_new_with_label ("Ok");
+  button = gtk_button_new_with_label ("Ok");
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (report_window)->action_area),
-    ok_button, TRUE, FALSE, 10);
-  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
+    button, TRUE, FALSE, 2);
+  gtk_box_set_spacing ( GTK_BOX (GTK_DIALOG (report_window)->action_area),
+    2 );
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
     GTK_SIGNAL_FUNC (ok_callback), rd);
-  GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);
-  gtk_widget_grab_default (ok_button);
-  gtk_widget_show (ok_button);
-  /*gtk_tooltips_set_tips (tooltips, ok_button,
-    "Generate a report" );*/
+  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+  gtk_widget_grab_default (button);
+  gtk_widget_show (button);
+  /*gtk_tooltips_set_tips (tooltips, button, "Generate a report" );*/
 
-  cancel_button = gtk_button_new_with_label ("Cancel");
+  button = gtk_button_new_with_label ("Cancel");
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (report_window)->action_area),
-    cancel_button, TRUE, FALSE, 10);
-  gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
+    button, TRUE, FALSE, 2);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
     GTK_SIGNAL_FUNC (cancel_callback), rd);
-  gtk_widget_show (cancel_button);
-  /*gtk_tooltips_set_tips (tooltips, cancel_button,
+  gtk_widget_show (button);
+  /*gtk_tooltips_set_tips (tooltips, button,
     "Cancel without generating a report" );*/
 
+  button = gtk_button_new_with_label ("Select all");
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (report_window)->action_area),
+    button, TRUE, FALSE, 2);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+    GTK_SIGNAL_FUNC (select_all_callback), rd);
+  gtk_widget_show (button);
+  /*gtk_tooltips_set_tips (tooltips, button, "Select all tasks" );*/
+
+  button = gtk_button_new_with_label ("Select none");
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (report_window)->action_area),
+    button, TRUE, FALSE, 2);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+    GTK_SIGNAL_FUNC (select_none_callback), rd);
+  gtk_widget_show (button);
+  /*gtk_tooltips_set_tips (tooltips, button, "Unselect all tasks" );*/
+
   gtk_widget_show (report_window);
 
   return ( report_window );
